#include "stdafx.h"
#include "stdlib.h"
#include "string.h"
#include "sys/stat.h"

/**
 * UTF16to8. Console application for UTF-16 to UTF-8 conversion.
 * Conversion table:
 *    UTF-16             UTF-8
 * 1) 00000yyy yyxxxxxx  110yyyyy 10xxxxxx
 * 2) zzzzyyyy yyxxxxxx  1110zzzz 10yyyyyy 10xxxxxx
 * Usage:
 * UTF16to8 input_file output_file
 * Cuckoo 0.1.
 * Copyright (c) 2001-2002 Alexis Grandemange
 * Mail: alexis.grandemange@pagebox.net
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version
 * 2.1 of the License.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * A copy of the GNU Lesser General Public License lesser.txt should be
 * included in the distribution.
 */

int processLine(char *ibuf, int bufsize, FILE *ifd, FILE *ofd) {
	if (feof(ifd) > 0)
		return 1;
	if (fgets(ibuf, bufsize, ifd) == NULL) {
		return -1;
	}
	char *ebuf = ibuf + strlen(ibuf);
	char *pos = strstr(ibuf, "&#");
	char *oldpos = ibuf;
	char charbuf[5];
	charbuf[4] = '\0';
	int code = 0;
	while(pos != NULL) {
		if (fwrite(oldpos, pos - oldpos, 1, ofd) == -1) {
			printf("UTF16to8 unable to write (%d, %s)\n", errno, strerror(errno));
			return -1;
		}
		if (pos[5] == ';') { // 3 figures
			memcpy(charbuf, pos + 2, 3);
			charbuf[3] = '\0';
			oldpos = pos + 6;
		} else {
			memcpy(charbuf, pos + 2, 4);
			oldpos = pos + 7;
		}
		sscanf(charbuf, "%d", &code);
		if (code < 256)		// not Unicode - keep the entity representation
			fwrite(pos, oldpos - pos, 1, ofd);
		else
		if (code < 2048) {		// Upper byte with 3 bits or less set
			int upper = code >> 6;
			upper += 0xc0;		// 11000000
			char bupper = (char)upper;
			int lower = code & 0x3f;
			lower += 0x80;		// 10000000
			char blower = (char)lower;
			// For little endian
			fwrite(&bupper, 1, 1, ofd);
			fwrite(&blower, 1, 1, ofd);
		}
		else {
			int supper = code >> 12;
			supper += 0xe0;		// 11100000
			char bsupper = (char)supper;
			int upper = code & 0xfc0;
			upper >>= 6;
			upper += 0x80;		// 10000000
			char bupper = (char)upper;
			int lower = code & 0x3f;
			lower += 0x80;		// 10000000
			char blower = (char)lower;
			// For little endian
			fwrite(&bsupper, 1, 1, ofd);
			fwrite(&bupper, 1, 1, ofd);
			fwrite(&blower, 1, 1, ofd);
		}
		pos = strstr(oldpos, "&#");
	}
	fwrite(oldpos, ebuf - oldpos, 1, ofd);
	return 0;
}
int main(int argc, char* argv[])
{
	if (argc != 3) {
		printf("Usage: UTF16to8 input_file output_file\n");
		return -1;
	}
	struct stat stbuf;
	if (stat(argv[1], &stbuf) == -1) {
		printf("UTF16to8: %s doesn't exist (%d, %s)\n", argv[1], errno, strerror(errno));
		return -1;
	}
	FILE *ifd = fopen(argv[1], "r");
	if (ifd == NULL) {
		printf("UTF16to8 unable to open %s (%d, %s)\n", argv[1], errno, strerror(errno));
		return -1;
	}
	FILE *ofd = fopen(argv[2], "w+");
	if (ofd == NULL) {
		printf("UTF16to8 unable to open %s (%d, %s)\n", argv[2], errno, strerror(errno));
		return -1;
	}
	char *ibuf = (char *)malloc(stbuf.st_size);
	if (ibuf == NULL) {
		printf("UTF16to8 unable to alloc %d bytes (%d, %s)\n", stbuf.st_size, errno, strerror(errno));
		return -1;
	}
	int rc = 0;
	while(rc == 0) {
		rc = processLine(ibuf, stbuf.st_size, ifd, ofd);
	}
	fclose(ifd);
	fclose(ofd);
	return rc;
}

