// MathGreek.cpp : Implementation of CMathGreek
#include "stdafx.h"
#include "Math.h"
#include "MathGreek.h"
#include "comdef.h"

/**
 * CMathGreek. COM component handling special character conversion and processing.
 * Cuckoo 0.1.5.
 * Copyright (c) 2001-2002 Alexis Grandemange
 * Mail: alexis.grandemange@pagebox.net
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version
 * 2.1 of the License.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * A copy of the GNU Lesser General Public License lesser.txt should be
 * included in the distribution.
 */

/**
 * Mathematics and Greek character detection and translation into html entities.
 * Called by Cuckoo. Uses the mgCodes table built in the constructor.
 * Processes strings of one character.
 */
STDMETHODIMP CMathGreek::convert(BSTR from, BSTR *to)
{
	WCHAR *p = (WCHAR *)from;
	short wc = (short)p[0];
	int ic = (int)wc;
	if (ic < 0)
		ic = ic + 4096;
	USES_CONVERSION;
	if ((ic < 0) || (ic > 256))
		*to = SysAllocString(T2OLE(""));
	else {
		char *resp = mgCodes[ic];
		if (*resp == NULL) {		// Not defined in mgCodes
			unsigned short uwc[2];
			uwc[0] = (unsigned short)ic;
			uwc[1] = 0;				// Ending zero
			*to = SysAllocString(uwc);
		} else
			*to = SysAllocString(T2OLE(resp));
	}
	SysFreeString(from);
	return S_OK;
}

/**
 * ISO-8859-1 character detection and processing.
 * Called by Cuckoo. Uses the normCodes table built in the constructor.
 * Processes strings of one character.
 */
STDMETHODIMP CMathGreek::normalize(BSTR from, BSTR *to)
{
	WCHAR *p = (WCHAR *)from;
	short wc = (short)p[0];
	int ic = (int)wc;
	if (ic < 0)
		ic = ic + 4096;
	USES_CONVERSION;
	if (ic < 0) {
		*to = SysAllocString(T2OLE(""));		// Shouldn't happen
		SysFreeString(from);
		return S_OK;
	}
	if (ic > 255) {		// Unicode
		if (ic == 8230) {
			// For Opera 5 that displays 8230 as a single dot.
			// Once again if you think that this code is useless complex remember that:
			// a) Depending on the case Word stores in Unicode or not
			// b) Depending on its editor and version browsers handle a Unicode character in a way or another
			// c) MSXML replaces entity codes by something that can be ascii
			*to = SysAllocString(T2OLE(normCodes[133]));
		} else {
			char buf[10];
			sprintf(buf, "!#%d;", ic);		// Protects the entity code. !# replaced by &# by cuckoo-gen.js
			*to = SysAllocString(T2OLE(buf));
		}
	} else {
		char *resp = normCodes[ic];
		if (resp == NULL) {
			unsigned short uwc[2];
			uwc[0] = (unsigned short)ic;
			uwc[1] = 0;
			*to = SysAllocString(uwc);
		} else
			*to = SysAllocString(T2OLE(resp));
	}
	SysFreeString(from);
	return S_OK;
}
