// MathGreek.h : Declaration of the CMathGreek

#ifndef __MATHGREEK_H_
#define __MATHGREEK_H_

#include "resource.h"       // main symbols
#include "wtypes.h"
#include "stdio.h"

/**
 * CMathGreek. COM component handling special character conversion and processing.
 * Cuckoo 0.1.5.
 * Copyright (c) 2001-2002 Alexis Grandemange
 * Mail: alexis.grandemange@pagebox.net
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version
 * 2.1 of the License.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * A copy of the GNU Lesser General Public License lesser.txt should be
 * included in the distribution.
 */

class ATL_NO_VTABLE CMathGreek : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMathGreek, &CLSID_MathGreek>,
	public IDispatchImpl<IMathGreek, &IID_IMathGreek, &LIBID_MATHLib>
{
	/** Mathematics and greek symbol processing */
	char *mgCodes[256];
	/** ISO-8859-1 symbol processing */
	char *normCodes[256];
public:
	/** Constructor. Populates mgCodes and normCodes. */
	CMathGreek()
	{
		for (int i = 0; i < 256; ++ i)
			mgCodes[i] = NULL;
		mgCodes[182] = "!#8706;";
		mgCodes[213] = "!#8719;";
		mgCodes[229] = "!#8721;";
		mgCodes[214] = "!#8730;";
		mgCodes[165] = "!#8734;";
		mgCodes[199] = "!#8745;";
		mgCodes[200] = "!#8746;";
		mgCodes[166] = "!#8747;";
		mgCodes[187] = "!#8776;";
		mgCodes[185] = "!#8800;";
		mgCodes[186] = "!#8801;";
		mgCodes[163] = "!#8804;";
		mgCodes[179] = "!#8805;";
		mgCodes[34] = "!#8704;";
		mgCodes[36] = "!#8707;";
		mgCodes[201] = "!#8835;";
		mgCodes[202] = "!#8839;";
		mgCodes[203] = "!#8836;";
		mgCodes[204] = "!#8834;";
		mgCodes[205] = "!#8838;";
		mgCodes[206] = "!#8712;";
		mgCodes[207] = "!#8713;";
		mgCodes[97] = "!#945;";
		mgCodes[98] = "!#946;";
		mgCodes[103] = "!#947;";
		mgCodes[100] = "!#948;";
		mgCodes[101] = "!#949;";
		mgCodes[122] = "!#950;";
		mgCodes[104] = "!#951;";
		mgCodes[113] = "!#952;";
		mgCodes[105] = "!#953;";
		mgCodes[107] = "!#954;";
		mgCodes[108] = "!#955;";
		mgCodes[109] = "!#956;";
		mgCodes[110] = "!#957;";
		mgCodes[120] = "!#958;";
		mgCodes[111] = "!#959;";
		mgCodes[112] = "!#960;";
		mgCodes[114] = "!#961;";
		mgCodes[86] = "!#962;";
		mgCodes[115] = "!#963;";
		mgCodes[116] = "!#964;";
		mgCodes[117] = "!#965;";
		mgCodes[102] = "!#966;";
		mgCodes[99] = "!#967;";
		mgCodes[121] = "!#968;";
		mgCodes[119] = "!#969;";
		mgCodes[65] = "!#913;";
		mgCodes[66] = "!#914;";
		mgCodes[71] = "!#915;";
		mgCodes[68] = "!#916;";
		mgCodes[69] = "!#917;";
		mgCodes[90] = "!#918;";
		mgCodes[72] = "!#919;";
		mgCodes[81] = "!#920;";
		mgCodes[73] = "!#921;";
		mgCodes[75] = "!#922;";
		mgCodes[76] = "!#923;";
		mgCodes[77] = "!#924;";
		mgCodes[78] = "!#925;";
		mgCodes[88] = "!#926;";
		mgCodes[79] = "!#927;";
		mgCodes[80] = "!#928;";
		mgCodes[82] = "!#929;";
		mgCodes[83] = "!#931;";
		mgCodes[84] = "!#932;";
		mgCodes[85] = "!#933;";
		mgCodes[70] = "!#934;";
		mgCodes[67] = "!#935;";
		mgCodes[89] = "!#936;";
		mgCodes[87] = "!#937;";
		for (i = 0; i < 256; ++ i)
			normCodes[i] = NULL;
		normCodes[5] = "";
		normCodes[7] = "";
		normCodes[13] = "";
		normCodes[21] = "";
		normCodes[14] = "";
		normCodes[12] = "";
		normCodes[11] = " ";
		normCodes[31] = " ";
		normCodes[133] = "...";
		normCodes[145] = "'";
		normCodes[146] = "'";
		normCodes[147] = "\"";
		normCodes[148] = "\"";
		normCodes[150] = "-";
		normCodes[151] = "-";
		normCodes[153] = "<b>(tm)</b>";
		normCodes[160] = "&#160;";
		normCodes[161] = "&#161;";
		normCodes[162] = "&#162;";
		normCodes[163] = "&#163;";
		normCodes[164] = "&#164;";
		normCodes[165] = "&#165;";
		normCodes[166] = "&#166;";
		normCodes[167] = "&#167;";
		normCodes[168] = "&#168;";
		normCodes[169] = "<b>(c)</b>";
		normCodes[170] = "&#170;";
		normCodes[171] = "&#171;";
		normCodes[172] = "&#172;";
		normCodes[173] = "&#173;";
		normCodes[174] = "<b>(r)</b>";
		normCodes[175] = "&#175;";
		normCodes[176] = "&#176;";
		normCodes[177] = "&#177;";
		normCodes[178] = "&#178;";
		normCodes[179] = "&#179;";
		normCodes[180] = "&#180;";
		normCodes[181] = "&#181;";
		normCodes[182] = "&#182;";
		normCodes[183] = "&#183;";
		normCodes[184] = "&#184;";
		normCodes[185] = "&#185;";
		normCodes[186] = "&#186;";
		normCodes[187] = "&#187;";
		normCodes[188] = "1/4";
		normCodes[189] = "1/2";
		normCodes[190] = "3/4";
		normCodes[191] = "&#191;";
		normCodes[192] = "&#192;";
		normCodes[193] = "&#193;";
		normCodes[194] = "&#194;";
		normCodes[195] = "&#195;";
		normCodes[196] = "&#196;";
		normCodes[197] = "&#197;";
		normCodes[198] = "&#198;";
		normCodes[199] = "&#199;";
		normCodes[200] = "&#200;";
		normCodes[201] = "&#201;";
		normCodes[202] = "&#202;";
		normCodes[203] = "&#203;";
		normCodes[204] = "&#204;";
		normCodes[205] = "&#205;";
		normCodes[206] = "&#206;";
		normCodes[207] = "&#207;";
		normCodes[208] = "&#208;";
		normCodes[209] = "&#209;";
		normCodes[210] = "&#210;";
		normCodes[211] = "&#211;";
		normCodes[212] = "&#212;";
		normCodes[213] = "&#213;";
		normCodes[214] = "&#214;";
		normCodes[215] = "x";
		normCodes[216] = "&#216;";
		normCodes[217] = "&#217;";
		normCodes[218] = "&#218;";
		normCodes[219] = "&#219;";
		normCodes[220] = "&#220;";
		normCodes[221] = "&#221;";
		normCodes[222] = "&#222;";
		normCodes[223] = "&#223;";
		normCodes[224] = "&#224;";
		normCodes[225] = "&#225;";
		normCodes[226] = "&#226;";
		normCodes[227] = "&#227;";
		normCodes[228] = "&#228;";
		normCodes[229] = "&#229;";
		normCodes[230] = "&#230;";
		normCodes[231] = "&#231;";
		normCodes[232] = "&#232;";
		normCodes[233] = "&#233;";
		normCodes[234] = "&#234;";
		normCodes[235] = "&#235;";
		normCodes[236] = "&#236;";
		normCodes[237] = "&#237;";
		normCodes[238] = "&#238;";
		normCodes[239] = "&#239;";
		normCodes[240] = "&#240;";
		normCodes[241] = "&#241;";
		normCodes[242] = "&#242;";
		normCodes[243] = "&#243;";
		normCodes[244] = "&#244;";
		normCodes[245] = "&#245;";
		normCodes[246] = "&#246;";
		normCodes[247] = "/";
		normCodes[248] = "&#248;";
		normCodes[249] = "&#249;";
		normCodes[250] = "&#250;";
		normCodes[251] = "&#251;";
		normCodes[252] = "&#252;";
		normCodes[253] = "&#253;";
		normCodes[254] = "&#254;";
		normCodes[255] = "&#255;";
	}

DECLARE_REGISTRY_RESOURCEID(IDR_MATHGREEK)
DECLARE_NOT_AGGREGATABLE(CMathGreek)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMathGreek)
	COM_INTERFACE_ENTRY(IMathGreek)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

// IMathGreek
public:
	STDMETHOD(normalize)(/*[in]*/ BSTR from, /*[out, retval]*/ BSTR *to);
	STDMETHOD(convert)(/*[in]*/ BSTR from, /*[out, retval]*/ BSTR *to);
};

#endif //__MATHGREEK_H_
